package io.hmit.modules.serviceOrder.controller;

import io.hmit.common.annotation.LogOperation;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.ExcelUtils;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.serviceOrder.dto.PensionCommunityDTO;
import io.hmit.modules.serviceOrder.excel.PensionCommunityExcel;
import io.hmit.modules.serviceOrder.service.PensionCommunityService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 社区表
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-02-03
 */
@RestController
@RequestMapping("serviceOrder/pensioncommunity")
@Api(tags="社区表")
public class PensionCommunityController {
    @Autowired
    private PensionCommunityService pensionCommunityService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
    @RequiresPermissions("serviceOrder:pensioncommunity:page")
    public Result<PageData<PensionCommunityDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<PensionCommunityDTO> page = pensionCommunityService.page(params);

        return new Result<PageData<PensionCommunityDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @RequiresPermissions("serviceOrder:pensioncommunity:info")
    public Result<PensionCommunityDTO> get(@PathVariable("id") Long id){
        PensionCommunityDTO data = pensionCommunityService.get(id);

        return new Result<PensionCommunityDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @RequiresPermissions("serviceOrder:pensioncommunity:save")
    public Result save(@RequestBody PensionCommunityDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        pensionCommunityService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @RequiresPermissions("serviceOrder:pensioncommunity:update")
    public Result update(@RequestBody PensionCommunityDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        pensionCommunityService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @RequiresPermissions("serviceOrder:pensioncommunity:delete")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        pensionCommunityService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @RequiresPermissions("serviceOrder:pensioncommunity:export")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<PensionCommunityDTO> list = pensionCommunityService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, PensionCommunityExcel.class);
    }

}