package io.hmit.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.api.WxConsts;
import me.chanjar.weixin.common.exception.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.result.WxMpOAuth2AccessToken;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.net.URLEncoder;

/**
 * @Description :
 * @Author : Shen Yuanfeng
 * @Date: 2021/3/17 15:32
 */

@Slf4j
@Controller
@RequestMapping("/wechat")
@Api(tags = "微信登录接口")
public class WeChatController {

    @Autowired
    private WxMpService wxMpService;

    @GetMapping("/authorize")
    @ApiOperation(value = "请求微信授权code", notes = "当returnUrl传入时，验证后将跳转到此路径")
    public String authorize(@RequestParam(value = "returnUrl", required = false) String returnUrl) {
        if (null == returnUrl) {
            returnUrl = "https://byyl.zjhmit.com/hmit-api/api/login";
        }
        String url = "https://byyl.zjhmit.com/hmit-api/wechat/authcode";
        String redirectUrl = wxMpService.oauth2buildAuthorizationUrl(url, WxConsts.OAUTH2_SCOPE_BASE, URLEncoder.encode(returnUrl));
        log.info("【微信网页授权】获取code，redirectUrl={}", redirectUrl);
        return "redirect:" + redirectUrl;
    }

    @GetMapping("/authcode")
    public String userInfo(@RequestParam("code") String code,
                           @RequestParam("state") String returnUrl) {
        log.info("【微信网页授权】code={}, returnUrl={}", code, returnUrl);
        WxMpOAuth2AccessToken wxMpOAuth2AccessToken = new WxMpOAuth2AccessToken();
        try {
            wxMpOAuth2AccessToken = wxMpService.oauth2getAccessToken(code);
        }catch (WxErrorException e){
            log.error("【微信网页授权】错误{}",e.getMessage());
        }
        String openid = wxMpOAuth2AccessToken.getOpenId();
        return "redirect:" + returnUrl + "?openid=" + openid;
    }

}
